#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
# define _cffi_float_complex_t   _Fcomplex    /* include <complex.h> for it */
# define _cffi_double_complex_t  _Dcomplex    /* include <complex.h> for it */
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
# define _cffi_float_complex_t   float _Complex
# define _cffi_double_complex_t  double _Complex
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/


                      #include <geopm_pio.h>
                      #include <geopm_error.h>
                      #include <geopm_shmem.h>
                      #include <geopm_topo.h>
                      #include <geopm_hash.h>
                      #include <geopm_stats_collector.h>
                      #include <geopm_access.h>
                      

/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(char const *)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 126), // char const *
/*  2 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  3 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(char const *, int *, int *, int *)
/*  4 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  5 */ _CFFI_OP(_CFFI_OP_POINTER, 11), // int *
/*  6 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/*  7 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/*  8 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  9 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(char const *, int, int)
/* 10 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 11 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7), // int
/* 12 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 13 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 14 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(char const *, int, int, double *)
/* 15 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 16 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 17 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 18 */ _CFFI_OP(_CFFI_OP_POINTER, 24), // double *
/* 19 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 20 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(char const *, int, int, double)
/* 21 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 22 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 23 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 24 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14), // double
/* 25 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 26 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(char const *, int, int, size_t, char *)
/* 27 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 28 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 29 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 30 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28), // size_t
/* 31 */ _CFFI_OP(_CFFI_OP_POINTER, 126), // char *
/* 32 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 33 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(char const *, size_t, char *)
/* 34 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 35 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 36 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 37 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 38 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(char const *, size_t, int, int, int)
/* 39 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 40 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 41 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 42 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 43 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 44 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 45 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(double, int, size_t, char *)
/* 46 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14),
/* 47 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 48 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 49 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 50 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 51 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(int)
/* 52 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 53 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 54 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(int, double *)
/* 55 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 56 */ _CFFI_OP(_CFFI_OP_NOOP, 18),
/* 57 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 58 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(int, double)
/* 59 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 60 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14),
/* 61 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 62 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(int, int)
/* 63 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 64 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 65 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 66 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(int, int, int, size_t, int *)
/* 67 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 68 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 69 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 70 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 71 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 72 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 73 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(int, int, struct geopm_request_s const *, int, struct geopm_request_s const *, int *, int, char *)
/* 74 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 75 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 76 */ _CFFI_OP(_CFFI_OP_POINTER, 140), // struct geopm_request_s const *
/* 77 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 78 */ _CFFI_OP(_CFFI_OP_NOOP, 76),
/* 79 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 80 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 81 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 82 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 83 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(int, size_t, char *)
/* 84 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 85 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 86 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 87 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 88 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(size_t, char *)
/* 89 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 90 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 91 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 92 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(size_t, struct geopm_request_s const *, struct geopm_stats_collector_s * *)
/* 93 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 94 */ _CFFI_OP(_CFFI_OP_NOOP, 76),
/* 95 */ _CFFI_OP(_CFFI_OP_POINTER, 98), // struct geopm_stats_collector_s * *
/* 96 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 97 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(struct geopm_stats_collector_s *)
/* 98 */ _CFFI_OP(_CFFI_OP_POINTER, 141), // struct geopm_stats_collector_s *
/* 99 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 100 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(struct geopm_stats_collector_s const *, size_t *)
/* 101 */ _CFFI_OP(_CFFI_OP_POINTER, 141), // struct geopm_stats_collector_s const *
/* 102 */ _CFFI_OP(_CFFI_OP_POINTER, 30), // size_t *
/* 103 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 104 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(struct geopm_stats_collector_s const *, size_t *, char *)
/* 105 */ _CFFI_OP(_CFFI_OP_NOOP, 101),
/* 106 */ _CFFI_OP(_CFFI_OP_NOOP, 102),
/* 107 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 108 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 109 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(struct geopm_stats_collector_s const *, size_t, struct geopm_report_s *)
/* 110 */ _CFFI_OP(_CFFI_OP_NOOP, 101),
/* 111 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 112 */ _CFFI_OP(_CFFI_OP_POINTER, 139), // struct geopm_report_s *
/* 113 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 114 */ _CFFI_OP(_CFFI_OP_FUNCTION, 11), // int()(void)
/* 115 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 116 */ _CFFI_OP(_CFFI_OP_FUNCTION, 142), // uint64_t()(char const *)
/* 117 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 118 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 119 */ _CFFI_OP(_CFFI_OP_FUNCTION, 143), // void()(int, char *, size_t)
/* 120 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 121 */ _CFFI_OP(_CFFI_OP_NOOP, 31),
/* 122 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 123 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 124 */ _CFFI_OP(_CFFI_OP_FUNCTION, 143), // void()(void)
/* 125 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 126 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 2), // char
/* 127 */ _CFFI_OP(_CFFI_OP_ARRAY, 126), // char[255]
/* 128 */ (_cffi_opcode_t)(255),
/* 129 */ _CFFI_OP(_CFFI_OP_ARRAY, 24), // double[4]
/* 130 */ (_cffi_opcode_t)(4),
/* 131 */ _CFFI_OP(_CFFI_OP_ARRAY, 24), // double[7]
/* 132 */ (_cffi_opcode_t)(7),
/* 133 */ _CFFI_OP(_CFFI_OP_ENUM, 0), // enum geopm_domain_e
/* 134 */ _CFFI_OP(_CFFI_OP_ENUM, 1), // enum geopm_error_e
/* 135 */ _CFFI_OP(_CFFI_OP_ENUM, 2), // enum geopm_metric_stats_e
/* 136 */ _CFFI_OP(_CFFI_OP_ENUM, 3), // enum geopm_sample_stats_e
/* 137 */ _CFFI_OP(_CFFI_OP_POINTER, 138), // struct geopm_metric_stats_s *
/* 138 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 0), // struct geopm_metric_stats_s
/* 139 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 1), // struct geopm_report_s
/* 140 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 2), // struct geopm_request_s
/* 141 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 3), // struct geopm_stats_collector_s
/* 142 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24), // uint64_t
/* 143 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

static int _cffi_const_GEOPM_DOMAIN_INVALID(unsigned long long *o)
{
  int n = (GEOPM_DOMAIN_INVALID) <= 0;
  *o = (unsigned long long)((GEOPM_DOMAIN_INVALID) | 0);  /* check that GEOPM_DOMAIN_INVALID is an integer */
  return n;
}

static int _cffi_const_GEOPM_DOMAIN_BOARD(unsigned long long *o)
{
  int n = (GEOPM_DOMAIN_BOARD) <= 0;
  *o = (unsigned long long)((GEOPM_DOMAIN_BOARD) | 0);  /* check that GEOPM_DOMAIN_BOARD is an integer */
  return n;
}

static int _cffi_const_GEOPM_DOMAIN_PACKAGE(unsigned long long *o)
{
  int n = (GEOPM_DOMAIN_PACKAGE) <= 0;
  *o = (unsigned long long)((GEOPM_DOMAIN_PACKAGE) | 0);  /* check that GEOPM_DOMAIN_PACKAGE is an integer */
  return n;
}

static int _cffi_const_GEOPM_DOMAIN_CORE(unsigned long long *o)
{
  int n = (GEOPM_DOMAIN_CORE) <= 0;
  *o = (unsigned long long)((GEOPM_DOMAIN_CORE) | 0);  /* check that GEOPM_DOMAIN_CORE is an integer */
  return n;
}

static int _cffi_const_GEOPM_DOMAIN_CPU(unsigned long long *o)
{
  int n = (GEOPM_DOMAIN_CPU) <= 0;
  *o = (unsigned long long)((GEOPM_DOMAIN_CPU) | 0);  /* check that GEOPM_DOMAIN_CPU is an integer */
  return n;
}

static int _cffi_const_GEOPM_DOMAIN_MEMORY(unsigned long long *o)
{
  int n = (GEOPM_DOMAIN_MEMORY) <= 0;
  *o = (unsigned long long)((GEOPM_DOMAIN_MEMORY) | 0);  /* check that GEOPM_DOMAIN_MEMORY is an integer */
  return n;
}

static int _cffi_const_GEOPM_DOMAIN_PACKAGE_INTEGRATED_MEMORY(unsigned long long *o)
{
  int n = (GEOPM_DOMAIN_PACKAGE_INTEGRATED_MEMORY) <= 0;
  *o = (unsigned long long)((GEOPM_DOMAIN_PACKAGE_INTEGRATED_MEMORY) | 0);  /* check that GEOPM_DOMAIN_PACKAGE_INTEGRATED_MEMORY is an integer */
  return n;
}

static int _cffi_const_GEOPM_DOMAIN_NIC(unsigned long long *o)
{
  int n = (GEOPM_DOMAIN_NIC) <= 0;
  *o = (unsigned long long)((GEOPM_DOMAIN_NIC) | 0);  /* check that GEOPM_DOMAIN_NIC is an integer */
  return n;
}

static int _cffi_const_GEOPM_DOMAIN_PACKAGE_INTEGRATED_NIC(unsigned long long *o)
{
  int n = (GEOPM_DOMAIN_PACKAGE_INTEGRATED_NIC) <= 0;
  *o = (unsigned long long)((GEOPM_DOMAIN_PACKAGE_INTEGRATED_NIC) | 0);  /* check that GEOPM_DOMAIN_PACKAGE_INTEGRATED_NIC is an integer */
  return n;
}

static int _cffi_const_GEOPM_DOMAIN_GPU(unsigned long long *o)
{
  int n = (GEOPM_DOMAIN_GPU) <= 0;
  *o = (unsigned long long)((GEOPM_DOMAIN_GPU) | 0);  /* check that GEOPM_DOMAIN_GPU is an integer */
  return n;
}

static int _cffi_const_GEOPM_DOMAIN_PACKAGE_INTEGRATED_GPU(unsigned long long *o)
{
  int n = (GEOPM_DOMAIN_PACKAGE_INTEGRATED_GPU) <= 0;
  *o = (unsigned long long)((GEOPM_DOMAIN_PACKAGE_INTEGRATED_GPU) | 0);  /* check that GEOPM_DOMAIN_PACKAGE_INTEGRATED_GPU is an integer */
  return n;
}

static int _cffi_const_GEOPM_DOMAIN_GPU_CHIP(unsigned long long *o)
{
  int n = (GEOPM_DOMAIN_GPU_CHIP) <= 0;
  *o = (unsigned long long)((GEOPM_DOMAIN_GPU_CHIP) | 0);  /* check that GEOPM_DOMAIN_GPU_CHIP is an integer */
  return n;
}

static int _cffi_const_GEOPM_NUM_DOMAIN(unsigned long long *o)
{
  int n = (GEOPM_NUM_DOMAIN) <= 0;
  *o = (unsigned long long)((GEOPM_NUM_DOMAIN) | 0);  /* check that GEOPM_NUM_DOMAIN is an integer */
  return n;
}

static int _cffi_const_GEOPM_ERROR_RUNTIME(unsigned long long *o)
{
  int n = (GEOPM_ERROR_RUNTIME) <= 0;
  *o = (unsigned long long)((GEOPM_ERROR_RUNTIME) | 0);  /* check that GEOPM_ERROR_RUNTIME is an integer */
  return n;
}

static int _cffi_const_GEOPM_ERROR_LOGIC(unsigned long long *o)
{
  int n = (GEOPM_ERROR_LOGIC) <= 0;
  *o = (unsigned long long)((GEOPM_ERROR_LOGIC) | 0);  /* check that GEOPM_ERROR_LOGIC is an integer */
  return n;
}

static int _cffi_const_GEOPM_ERROR_INVALID(unsigned long long *o)
{
  int n = (GEOPM_ERROR_INVALID) <= 0;
  *o = (unsigned long long)((GEOPM_ERROR_INVALID) | 0);  /* check that GEOPM_ERROR_INVALID is an integer */
  return n;
}

static int _cffi_const_GEOPM_ERROR_FILE_PARSE(unsigned long long *o)
{
  int n = (GEOPM_ERROR_FILE_PARSE) <= 0;
  *o = (unsigned long long)((GEOPM_ERROR_FILE_PARSE) | 0);  /* check that GEOPM_ERROR_FILE_PARSE is an integer */
  return n;
}

static int _cffi_const_GEOPM_ERROR_LEVEL_RANGE(unsigned long long *o)
{
  int n = (GEOPM_ERROR_LEVEL_RANGE) <= 0;
  *o = (unsigned long long)((GEOPM_ERROR_LEVEL_RANGE) | 0);  /* check that GEOPM_ERROR_LEVEL_RANGE is an integer */
  return n;
}

static int _cffi_const_GEOPM_ERROR_NOT_IMPLEMENTED(unsigned long long *o)
{
  int n = (GEOPM_ERROR_NOT_IMPLEMENTED) <= 0;
  *o = (unsigned long long)((GEOPM_ERROR_NOT_IMPLEMENTED) | 0);  /* check that GEOPM_ERROR_NOT_IMPLEMENTED is an integer */
  return n;
}

static int _cffi_const_GEOPM_ERROR_PLATFORM_UNSUPPORTED(unsigned long long *o)
{
  int n = (GEOPM_ERROR_PLATFORM_UNSUPPORTED) <= 0;
  *o = (unsigned long long)((GEOPM_ERROR_PLATFORM_UNSUPPORTED) | 0);  /* check that GEOPM_ERROR_PLATFORM_UNSUPPORTED is an integer */
  return n;
}

static int _cffi_const_GEOPM_ERROR_MSR_OPEN(unsigned long long *o)
{
  int n = (GEOPM_ERROR_MSR_OPEN) <= 0;
  *o = (unsigned long long)((GEOPM_ERROR_MSR_OPEN) | 0);  /* check that GEOPM_ERROR_MSR_OPEN is an integer */
  return n;
}

static int _cffi_const_GEOPM_ERROR_MSR_READ(unsigned long long *o)
{
  int n = (GEOPM_ERROR_MSR_READ) <= 0;
  *o = (unsigned long long)((GEOPM_ERROR_MSR_READ) | 0);  /* check that GEOPM_ERROR_MSR_READ is an integer */
  return n;
}

static int _cffi_const_GEOPM_ERROR_MSR_WRITE(unsigned long long *o)
{
  int n = (GEOPM_ERROR_MSR_WRITE) <= 0;
  *o = (unsigned long long)((GEOPM_ERROR_MSR_WRITE) | 0);  /* check that GEOPM_ERROR_MSR_WRITE is an integer */
  return n;
}

static int _cffi_const_GEOPM_ERROR_AGENT_UNSUPPORTED(unsigned long long *o)
{
  int n = (GEOPM_ERROR_AGENT_UNSUPPORTED) <= 0;
  *o = (unsigned long long)((GEOPM_ERROR_AGENT_UNSUPPORTED) | 0);  /* check that GEOPM_ERROR_AGENT_UNSUPPORTED is an integer */
  return n;
}

static int _cffi_const_GEOPM_ERROR_AFFINITY(unsigned long long *o)
{
  int n = (GEOPM_ERROR_AFFINITY) <= 0;
  *o = (unsigned long long)((GEOPM_ERROR_AFFINITY) | 0);  /* check that GEOPM_ERROR_AFFINITY is an integer */
  return n;
}

static int _cffi_const_GEOPM_ERROR_NO_AGENT(unsigned long long *o)
{
  int n = (GEOPM_ERROR_NO_AGENT) <= 0;
  *o = (unsigned long long)((GEOPM_ERROR_NO_AGENT) | 0);  /* check that GEOPM_ERROR_NO_AGENT is an integer */
  return n;
}

static int _cffi_const_GEOPM_METRIC_COUNT(unsigned long long *o)
{
  int n = (GEOPM_METRIC_COUNT) <= 0;
  *o = (unsigned long long)((GEOPM_METRIC_COUNT) | 0);  /* check that GEOPM_METRIC_COUNT is an integer */
  return n;
}

static int _cffi_const_GEOPM_METRIC_FIRST(unsigned long long *o)
{
  int n = (GEOPM_METRIC_FIRST) <= 0;
  *o = (unsigned long long)((GEOPM_METRIC_FIRST) | 0);  /* check that GEOPM_METRIC_FIRST is an integer */
  return n;
}

static int _cffi_const_GEOPM_METRIC_LAST(unsigned long long *o)
{
  int n = (GEOPM_METRIC_LAST) <= 0;
  *o = (unsigned long long)((GEOPM_METRIC_LAST) | 0);  /* check that GEOPM_METRIC_LAST is an integer */
  return n;
}

static int _cffi_const_GEOPM_METRIC_MIN(unsigned long long *o)
{
  int n = (GEOPM_METRIC_MIN) <= 0;
  *o = (unsigned long long)((GEOPM_METRIC_MIN) | 0);  /* check that GEOPM_METRIC_MIN is an integer */
  return n;
}

static int _cffi_const_GEOPM_METRIC_MAX(unsigned long long *o)
{
  int n = (GEOPM_METRIC_MAX) <= 0;
  *o = (unsigned long long)((GEOPM_METRIC_MAX) | 0);  /* check that GEOPM_METRIC_MAX is an integer */
  return n;
}

static int _cffi_const_GEOPM_METRIC_MEAN(unsigned long long *o)
{
  int n = (GEOPM_METRIC_MEAN) <= 0;
  *o = (unsigned long long)((GEOPM_METRIC_MEAN) | 0);  /* check that GEOPM_METRIC_MEAN is an integer */
  return n;
}

static int _cffi_const_GEOPM_METRIC_STD(unsigned long long *o)
{
  int n = (GEOPM_METRIC_STD) <= 0;
  *o = (unsigned long long)((GEOPM_METRIC_STD) | 0);  /* check that GEOPM_METRIC_STD is an integer */
  return n;
}

static int _cffi_const_GEOPM_NUM_METRIC_STATS(unsigned long long *o)
{
  int n = (GEOPM_NUM_METRIC_STATS) <= 0;
  *o = (unsigned long long)((GEOPM_NUM_METRIC_STATS) | 0);  /* check that GEOPM_NUM_METRIC_STATS is an integer */
  return n;
}

static int _cffi_const_GEOPM_SAMPLE_TIME_TOTAL(unsigned long long *o)
{
  int n = (GEOPM_SAMPLE_TIME_TOTAL) <= 0;
  *o = (unsigned long long)((GEOPM_SAMPLE_TIME_TOTAL) | 0);  /* check that GEOPM_SAMPLE_TIME_TOTAL is an integer */
  return n;
}

static int _cffi_const_GEOPM_SAMPLE_COUNT(unsigned long long *o)
{
  int n = (GEOPM_SAMPLE_COUNT) <= 0;
  *o = (unsigned long long)((GEOPM_SAMPLE_COUNT) | 0);  /* check that GEOPM_SAMPLE_COUNT is an integer */
  return n;
}

static int _cffi_const_GEOPM_SAMPLE_PERIOD_MEAN(unsigned long long *o)
{
  int n = (GEOPM_SAMPLE_PERIOD_MEAN) <= 0;
  *o = (unsigned long long)((GEOPM_SAMPLE_PERIOD_MEAN) | 0);  /* check that GEOPM_SAMPLE_PERIOD_MEAN is an integer */
  return n;
}

static int _cffi_const_GEOPM_SAMPLE_PERIOD_STD(unsigned long long *o)
{
  int n = (GEOPM_SAMPLE_PERIOD_STD) <= 0;
  *o = (unsigned long long)((GEOPM_SAMPLE_PERIOD_STD) | 0);  /* check that GEOPM_SAMPLE_PERIOD_STD is an integer */
  return n;
}

static int _cffi_const_GEOPM_NUM_SAMPLE_STATS(unsigned long long *o)
{
  int n = (GEOPM_NUM_SAMPLE_STATS) <= 0;
  *o = (unsigned long long)((GEOPM_NUM_SAMPLE_STATS) | 0);  /* check that GEOPM_NUM_SAMPLE_STATS is an integer */
  return n;
}

static uint64_t _cffi_d_geopm_crc32_str(char const * x0)
{
  return geopm_crc32_str(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_crc32_str(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  uint64_t result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_crc32_str(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, uint64_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_crc32_str _cffi_d_geopm_crc32_str
#endif

static void _cffi_d_geopm_error_message(int x0, char * x1, size_t x2)
{
  geopm_error_message(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_error_message(PyObject *self, PyObject *args)
{
  int x0;
  char * x1;
  size_t x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "geopm_error_message", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, size_t);
  if (x2 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { geopm_error_message(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_geopm_error_message _cffi_d_geopm_error_message
#endif

static int _cffi_d_geopm_msr_allowlist(size_t x0, char * x1)
{
  return geopm_msr_allowlist(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_msr_allowlist(PyObject *self, PyObject *args)
{
  size_t x0;
  char * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "geopm_msr_allowlist", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, size_t);
  if (x0 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_msr_allowlist(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_msr_allowlist _cffi_d_geopm_msr_allowlist
#endif

static int _cffi_d_geopm_pio_adjust(int x0, double x1)
{
  return geopm_pio_adjust(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_adjust(PyObject *self, PyObject *args)
{
  int x0;
  double x1;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "geopm_pio_adjust", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = (double)_cffi_to_c_double(arg1);
  if (x1 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_adjust(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_adjust _cffi_d_geopm_pio_adjust
#endif

static int _cffi_d_geopm_pio_control_description(char const * x0, size_t x1, char * x2)
{
  return geopm_pio_control_description(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_control_description(PyObject *self, PyObject *args)
{
  char const * x0;
  size_t x1;
  char * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "geopm_pio_control_description", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, size_t);
  if (x1 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_control_description(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_control_description _cffi_d_geopm_pio_control_description
#endif

static int _cffi_d_geopm_pio_control_domain_type(char const * x0)
{
  return geopm_pio_control_domain_type(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_control_domain_type(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_control_domain_type(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_control_domain_type _cffi_d_geopm_pio_control_domain_type
#endif

static int _cffi_d_geopm_pio_control_name(int x0, size_t x1, char * x2)
{
  return geopm_pio_control_name(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_control_name(PyObject *self, PyObject *args)
{
  int x0;
  size_t x1;
  char * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "geopm_pio_control_name", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, size_t);
  if (x1 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_control_name(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_control_name _cffi_d_geopm_pio_control_name
#endif

static int _cffi_d_geopm_pio_format_signal(double x0, int x1, size_t x2, char * x3)
{
  return geopm_pio_format_signal(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_format_signal(PyObject *self, PyObject *args)
{
  double x0;
  int x1;
  size_t x2;
  char * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "geopm_pio_format_signal", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  x0 = (double)_cffi_to_c_double(arg0);
  if (x0 == (double)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, size_t);
  if (x2 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_format_signal(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_format_signal _cffi_d_geopm_pio_format_signal
#endif

static int _cffi_d_geopm_pio_num_control_name(void)
{
  return geopm_pio_num_control_name();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_num_control_name(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_num_control_name(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_num_control_name _cffi_d_geopm_pio_num_control_name
#endif

static int _cffi_d_geopm_pio_num_signal_name(void)
{
  return geopm_pio_num_signal_name();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_num_signal_name(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_num_signal_name(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_num_signal_name _cffi_d_geopm_pio_num_signal_name
#endif

static int _cffi_d_geopm_pio_push_control(char const * x0, int x1, int x2)
{
  return geopm_pio_push_control(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_push_control(PyObject *self, PyObject *args)
{
  char const * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "geopm_pio_push_control", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_push_control(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_push_control _cffi_d_geopm_pio_push_control
#endif

static int _cffi_d_geopm_pio_push_signal(char const * x0, int x1, int x2)
{
  return geopm_pio_push_signal(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_push_signal(PyObject *self, PyObject *args)
{
  char const * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "geopm_pio_push_signal", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_push_signal(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_push_signal _cffi_d_geopm_pio_push_signal
#endif

static int _cffi_d_geopm_pio_read_batch(void)
{
  return geopm_pio_read_batch();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_read_batch(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_read_batch(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_read_batch _cffi_d_geopm_pio_read_batch
#endif

static int _cffi_d_geopm_pio_read_signal(char const * x0, int x1, int x2, double * x3)
{
  return geopm_pio_read_signal(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_read_signal(PyObject *self, PyObject *args)
{
  char const * x0;
  int x1;
  int x2;
  double * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "geopm_pio_read_signal", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(18), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(18), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_read_signal(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_read_signal _cffi_d_geopm_pio_read_signal
#endif

static void _cffi_d_geopm_pio_reset(void)
{
  geopm_pio_reset();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_reset(PyObject *self, PyObject *noarg)
{

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { geopm_pio_reset(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_geopm_pio_reset _cffi_d_geopm_pio_reset
#endif

static int _cffi_d_geopm_pio_restore_control(void)
{
  return geopm_pio_restore_control();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_restore_control(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_restore_control(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_restore_control _cffi_d_geopm_pio_restore_control
#endif

static int _cffi_d_geopm_pio_restore_control_dir(char const * x0)
{
  return geopm_pio_restore_control_dir(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_restore_control_dir(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_restore_control_dir(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_restore_control_dir _cffi_d_geopm_pio_restore_control_dir
#endif

static int _cffi_d_geopm_pio_sample(int x0, double * x1)
{
  return geopm_pio_sample(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_sample(PyObject *self, PyObject *args)
{
  int x0;
  double * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "geopm_pio_sample", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(18), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(18), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_sample(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_sample _cffi_d_geopm_pio_sample
#endif

static int _cffi_d_geopm_pio_save_control(void)
{
  return geopm_pio_save_control();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_save_control(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_save_control(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_save_control _cffi_d_geopm_pio_save_control
#endif

static int _cffi_d_geopm_pio_save_control_dir(char const * x0)
{
  return geopm_pio_save_control_dir(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_save_control_dir(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_save_control_dir(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_save_control_dir _cffi_d_geopm_pio_save_control_dir
#endif

static int _cffi_d_geopm_pio_signal_description(char const * x0, size_t x1, char * x2)
{
  return geopm_pio_signal_description(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_signal_description(PyObject *self, PyObject *args)
{
  char const * x0;
  size_t x1;
  char * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "geopm_pio_signal_description", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, size_t);
  if (x1 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_signal_description(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_signal_description _cffi_d_geopm_pio_signal_description
#endif

static int _cffi_d_geopm_pio_signal_domain_type(char const * x0)
{
  return geopm_pio_signal_domain_type(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_signal_domain_type(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_signal_domain_type(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_signal_domain_type _cffi_d_geopm_pio_signal_domain_type
#endif

static int _cffi_d_geopm_pio_signal_info(char const * x0, int * x1, int * x2, int * x3)
{
  return geopm_pio_signal_info(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_signal_info(PyObject *self, PyObject *args)
{
  char const * x0;
  int * x1;
  int * x2;
  int * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "geopm_pio_signal_info", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_signal_info(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_signal_info _cffi_d_geopm_pio_signal_info
#endif

static int _cffi_d_geopm_pio_signal_name(int x0, size_t x1, char * x2)
{
  return geopm_pio_signal_name(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_signal_name(PyObject *self, PyObject *args)
{
  int x0;
  size_t x1;
  char * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "geopm_pio_signal_name", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, size_t);
  if (x1 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_signal_name(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_signal_name _cffi_d_geopm_pio_signal_name
#endif

static int _cffi_d_geopm_pio_start_batch_server(int x0, int x1, struct geopm_request_s const * x2, int x3, struct geopm_request_s const * x4, int * x5, int x6, char * x7)
{
  return geopm_pio_start_batch_server(x0, x1, x2, x3, x4, x5, x6, x7);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_start_batch_server(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  struct geopm_request_s const * x2;
  int x3;
  struct geopm_request_s const * x4;
  int * x5;
  int x6;
  char * x7;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;

  if (!PyArg_UnpackTuple(args, "geopm_pio_start_batch_server", 8, 8, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(76), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (struct geopm_request_s const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(76), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(76), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (struct geopm_request_s const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(76), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x6 = _cffi_to_c_int(arg6, int);
  if (x6 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg7, (char **)&x7);
  if (datasize != 0) {
    x7 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg7, (char **)&x7,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_start_batch_server(x0, x1, x2, x3, x4, x5, x6, x7); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_start_batch_server _cffi_d_geopm_pio_start_batch_server
#endif

static int _cffi_d_geopm_pio_stop_batch_server(int x0)
{
  return geopm_pio_stop_batch_server(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_stop_batch_server(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_stop_batch_server(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_stop_batch_server _cffi_d_geopm_pio_stop_batch_server
#endif

static int _cffi_d_geopm_pio_write_batch(void)
{
  return geopm_pio_write_batch();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_write_batch(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_write_batch(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_write_batch _cffi_d_geopm_pio_write_batch
#endif

static int _cffi_d_geopm_pio_write_control(char const * x0, int x1, int x2, double x3)
{
  return geopm_pio_write_control(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_pio_write_control(PyObject *self, PyObject *args)
{
  char const * x0;
  int x1;
  int x2;
  double x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "geopm_pio_write_control", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = (double)_cffi_to_c_double(arg3);
  if (x3 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_pio_write_control(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_pio_write_control _cffi_d_geopm_pio_write_control
#endif

static int _cffi_d_geopm_shmem_create_prof(char const * x0, size_t x1, int x2, int x3, int x4)
{
  return geopm_shmem_create_prof(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_shmem_create_prof(PyObject *self, PyObject *args)
{
  char const * x0;
  size_t x1;
  int x2;
  int x3;
  int x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "geopm_shmem_create_prof", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, size_t);
  if (x1 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_shmem_create_prof(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_shmem_create_prof _cffi_d_geopm_shmem_create_prof
#endif

static int _cffi_d_geopm_shmem_path_prof(char const * x0, int x1, int x2, size_t x3, char * x4)
{
  return geopm_shmem_path_prof(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_shmem_path_prof(PyObject *self, PyObject *args)
{
  char const * x0;
  int x1;
  int x2;
  size_t x3;
  char * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "geopm_shmem_path_prof", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_shmem_path_prof(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_shmem_path_prof _cffi_d_geopm_shmem_path_prof
#endif

static int _cffi_d_geopm_stats_collector_create(size_t x0, struct geopm_request_s const * x1, struct geopm_stats_collector_s * * x2)
{
  return geopm_stats_collector_create(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_stats_collector_create(PyObject *self, PyObject *args)
{
  size_t x0;
  struct geopm_request_s const * x1;
  struct geopm_stats_collector_s * * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "geopm_stats_collector_create", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = _cffi_to_c_int(arg0, size_t);
  if (x0 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(76), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct geopm_request_s const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(76), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(95), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (struct geopm_stats_collector_s * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(95), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_stats_collector_create(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_stats_collector_create _cffi_d_geopm_stats_collector_create
#endif

static int _cffi_d_geopm_stats_collector_free(struct geopm_stats_collector_s * x0)
{
  return geopm_stats_collector_free(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_stats_collector_free(PyObject *self, PyObject *arg0)
{
  struct geopm_stats_collector_s * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(98), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct geopm_stats_collector_s *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(98), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_stats_collector_free(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_stats_collector_free _cffi_d_geopm_stats_collector_free
#endif

static int _cffi_d_geopm_stats_collector_report(struct geopm_stats_collector_s const * x0, size_t x1, struct geopm_report_s * x2)
{
  return geopm_stats_collector_report(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_stats_collector_report(PyObject *self, PyObject *args)
{
  struct geopm_stats_collector_s const * x0;
  size_t x1;
  struct geopm_report_s * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "geopm_stats_collector_report", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(101), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct geopm_stats_collector_s const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(101), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, size_t);
  if (x1 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(112), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (struct geopm_report_s *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(112), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_stats_collector_report(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_stats_collector_report _cffi_d_geopm_stats_collector_report
#endif

static int _cffi_d_geopm_stats_collector_report_yaml(struct geopm_stats_collector_s const * x0, size_t * x1, char * x2)
{
  return geopm_stats_collector_report_yaml(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_stats_collector_report_yaml(PyObject *self, PyObject *args)
{
  struct geopm_stats_collector_s const * x0;
  size_t * x1;
  char * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "geopm_stats_collector_report_yaml", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(101), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct geopm_stats_collector_s const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(101), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(102), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(102), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_stats_collector_report_yaml(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_stats_collector_report_yaml _cffi_d_geopm_stats_collector_report_yaml
#endif

static int _cffi_d_geopm_stats_collector_reset(struct geopm_stats_collector_s * x0)
{
  return geopm_stats_collector_reset(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_stats_collector_reset(PyObject *self, PyObject *arg0)
{
  struct geopm_stats_collector_s * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(98), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct geopm_stats_collector_s *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(98), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_stats_collector_reset(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_stats_collector_reset _cffi_d_geopm_stats_collector_reset
#endif

static int _cffi_d_geopm_stats_collector_update(struct geopm_stats_collector_s * x0)
{
  return geopm_stats_collector_update(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_stats_collector_update(PyObject *self, PyObject *arg0)
{
  struct geopm_stats_collector_s * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(98), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct geopm_stats_collector_s *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(98), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_stats_collector_update(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_stats_collector_update _cffi_d_geopm_stats_collector_update
#endif

static int _cffi_d_geopm_stats_collector_update_count(struct geopm_stats_collector_s const * x0, size_t * x1)
{
  return geopm_stats_collector_update_count(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_stats_collector_update_count(PyObject *self, PyObject *args)
{
  struct geopm_stats_collector_s const * x0;
  size_t * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "geopm_stats_collector_update_count", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(101), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct geopm_stats_collector_s const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(101), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(102), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(102), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_stats_collector_update_count(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_stats_collector_update_count _cffi_d_geopm_stats_collector_update_count
#endif

static int _cffi_d_geopm_topo_create_cache(void)
{
  return geopm_topo_create_cache();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_topo_create_cache(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_topo_create_cache(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_geopm_topo_create_cache _cffi_d_geopm_topo_create_cache
#endif

static int _cffi_d_geopm_topo_domain_idx(int x0, int x1)
{
  return geopm_topo_domain_idx(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_topo_domain_idx(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "geopm_topo_domain_idx", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_topo_domain_idx(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_geopm_topo_domain_idx _cffi_d_geopm_topo_domain_idx
#endif

static int _cffi_d_geopm_topo_domain_name(int x0, size_t x1, char * x2)
{
  return geopm_topo_domain_name(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_topo_domain_name(PyObject *self, PyObject *args)
{
  int x0;
  size_t x1;
  char * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "geopm_topo_domain_name", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, size_t);
  if (x1 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(31), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(31), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_topo_domain_name(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_topo_domain_name _cffi_d_geopm_topo_domain_name
#endif

static int _cffi_d_geopm_topo_domain_nested(int x0, int x1, int x2, size_t x3, int * x4)
{
  return geopm_topo_domain_nested(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_topo_domain_nested(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int x2;
  size_t x3;
  int * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "geopm_topo_domain_nested", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_topo_domain_nested(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_topo_domain_nested _cffi_d_geopm_topo_domain_nested
#endif

static int _cffi_d_geopm_topo_domain_type(char const * x0)
{
  return geopm_topo_domain_type(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_topo_domain_type(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_topo_domain_type(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_geopm_topo_domain_type _cffi_d_geopm_topo_domain_type
#endif

static int _cffi_d_geopm_topo_num_domain(int x0)
{
  return geopm_topo_num_domain(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_topo_num_domain(PyObject *self, PyObject *arg0)
{
  int x0;
  int result;
  PyObject *pyresult;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_topo_num_domain(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_geopm_topo_num_domain _cffi_d_geopm_topo_num_domain
#endif

static int _cffi_d_geopm_topo_num_domain_nested(int x0, int x1)
{
  return geopm_topo_num_domain_nested(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_geopm_topo_num_domain_nested(PyObject *self, PyObject *args)
{
  int x0;
  int x1;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "geopm_topo_num_domain_nested", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = _cffi_to_c_int(arg0, int);
  if (x0 == (int)-1 && PyErr_Occurred())
    return NULL;

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = geopm_topo_num_domain_nested(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_geopm_topo_num_domain_nested _cffi_d_geopm_topo_num_domain_nested
#endif

_CFFI_UNUSED_FN
static void _cffi_checkfld_struct_geopm_metric_stats_s(struct geopm_metric_stats_s *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char(*tmp)[255] = &p->name; (void)tmp; }
  { double(*tmp)[7] = &p->stats; (void)tmp; }
}
struct _cffi_align_struct_geopm_metric_stats_s { char x; struct geopm_metric_stats_s y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_struct_geopm_report_s(struct geopm_report_s *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char(*tmp)[255] = &p->host; (void)tmp; }
  { char(*tmp)[255] = &p->sample_time_first; (void)tmp; }
  { double(*tmp)[4] = &p->sample_stats; (void)tmp; }
  (void)((p->num_metric) | 0);  /* check that 'struct geopm_report_s.num_metric' is an integer */
  { struct geopm_metric_stats_s * *tmp = &p->metric_stats; (void)tmp; }
}
struct _cffi_align_struct_geopm_report_s { char x; struct geopm_report_s y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_struct_geopm_request_s(struct geopm_request_s *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->domain_type) | 0);  /* check that 'struct geopm_request_s.domain_type' is an integer */
  (void)((p->domain_idx) | 0);  /* check that 'struct geopm_request_s.domain_idx' is an integer */
  { char(*tmp)[255] = &p->name; (void)tmp; }
}
struct _cffi_align_struct_geopm_request_s { char x; struct geopm_request_s y; };

static const struct _cffi_global_s _cffi_globals[] = {
  { "GEOPM_DOMAIN_BOARD", (void *)_cffi_const_GEOPM_DOMAIN_BOARD, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_DOMAIN_CORE", (void *)_cffi_const_GEOPM_DOMAIN_CORE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_DOMAIN_CPU", (void *)_cffi_const_GEOPM_DOMAIN_CPU, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_DOMAIN_GPU", (void *)_cffi_const_GEOPM_DOMAIN_GPU, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_DOMAIN_GPU_CHIP", (void *)_cffi_const_GEOPM_DOMAIN_GPU_CHIP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_DOMAIN_INVALID", (void *)_cffi_const_GEOPM_DOMAIN_INVALID, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_DOMAIN_MEMORY", (void *)_cffi_const_GEOPM_DOMAIN_MEMORY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_DOMAIN_NIC", (void *)_cffi_const_GEOPM_DOMAIN_NIC, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_DOMAIN_PACKAGE", (void *)_cffi_const_GEOPM_DOMAIN_PACKAGE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_DOMAIN_PACKAGE_INTEGRATED_GPU", (void *)_cffi_const_GEOPM_DOMAIN_PACKAGE_INTEGRATED_GPU, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_DOMAIN_PACKAGE_INTEGRATED_MEMORY", (void *)_cffi_const_GEOPM_DOMAIN_PACKAGE_INTEGRATED_MEMORY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_DOMAIN_PACKAGE_INTEGRATED_NIC", (void *)_cffi_const_GEOPM_DOMAIN_PACKAGE_INTEGRATED_NIC, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_ERROR_AFFINITY", (void *)_cffi_const_GEOPM_ERROR_AFFINITY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_ERROR_AGENT_UNSUPPORTED", (void *)_cffi_const_GEOPM_ERROR_AGENT_UNSUPPORTED, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_ERROR_FILE_PARSE", (void *)_cffi_const_GEOPM_ERROR_FILE_PARSE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_ERROR_INVALID", (void *)_cffi_const_GEOPM_ERROR_INVALID, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_ERROR_LEVEL_RANGE", (void *)_cffi_const_GEOPM_ERROR_LEVEL_RANGE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_ERROR_LOGIC", (void *)_cffi_const_GEOPM_ERROR_LOGIC, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_ERROR_MSR_OPEN", (void *)_cffi_const_GEOPM_ERROR_MSR_OPEN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_ERROR_MSR_READ", (void *)_cffi_const_GEOPM_ERROR_MSR_READ, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_ERROR_MSR_WRITE", (void *)_cffi_const_GEOPM_ERROR_MSR_WRITE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_ERROR_NOT_IMPLEMENTED", (void *)_cffi_const_GEOPM_ERROR_NOT_IMPLEMENTED, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_ERROR_NO_AGENT", (void *)_cffi_const_GEOPM_ERROR_NO_AGENT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_ERROR_PLATFORM_UNSUPPORTED", (void *)_cffi_const_GEOPM_ERROR_PLATFORM_UNSUPPORTED, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_ERROR_RUNTIME", (void *)_cffi_const_GEOPM_ERROR_RUNTIME, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_METRIC_COUNT", (void *)_cffi_const_GEOPM_METRIC_COUNT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_METRIC_FIRST", (void *)_cffi_const_GEOPM_METRIC_FIRST, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_METRIC_LAST", (void *)_cffi_const_GEOPM_METRIC_LAST, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_METRIC_MAX", (void *)_cffi_const_GEOPM_METRIC_MAX, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_METRIC_MEAN", (void *)_cffi_const_GEOPM_METRIC_MEAN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_METRIC_MIN", (void *)_cffi_const_GEOPM_METRIC_MIN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_METRIC_STD", (void *)_cffi_const_GEOPM_METRIC_STD, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_NUM_DOMAIN", (void *)_cffi_const_GEOPM_NUM_DOMAIN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_NUM_METRIC_STATS", (void *)_cffi_const_GEOPM_NUM_METRIC_STATS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_NUM_SAMPLE_STATS", (void *)_cffi_const_GEOPM_NUM_SAMPLE_STATS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_SAMPLE_COUNT", (void *)_cffi_const_GEOPM_SAMPLE_COUNT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_SAMPLE_PERIOD_MEAN", (void *)_cffi_const_GEOPM_SAMPLE_PERIOD_MEAN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_SAMPLE_PERIOD_STD", (void *)_cffi_const_GEOPM_SAMPLE_PERIOD_STD, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "GEOPM_SAMPLE_TIME_TOTAL", (void *)_cffi_const_GEOPM_SAMPLE_TIME_TOTAL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "geopm_crc32_str", (void *)_cffi_f_geopm_crc32_str, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 116), (void *)_cffi_d_geopm_crc32_str },
  { "geopm_error_message", (void *)_cffi_f_geopm_error_message, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 119), (void *)_cffi_d_geopm_error_message },
  { "geopm_msr_allowlist", (void *)_cffi_f_geopm_msr_allowlist, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 88), (void *)_cffi_d_geopm_msr_allowlist },
  { "geopm_pio_adjust", (void *)_cffi_f_geopm_pio_adjust, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 58), (void *)_cffi_d_geopm_pio_adjust },
  { "geopm_pio_control_description", (void *)_cffi_f_geopm_pio_control_description, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 33), (void *)_cffi_d_geopm_pio_control_description },
  { "geopm_pio_control_domain_type", (void *)_cffi_f_geopm_pio_control_domain_type, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 0), (void *)_cffi_d_geopm_pio_control_domain_type },
  { "geopm_pio_control_name", (void *)_cffi_f_geopm_pio_control_name, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 83), (void *)_cffi_d_geopm_pio_control_name },
  { "geopm_pio_format_signal", (void *)_cffi_f_geopm_pio_format_signal, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 45), (void *)_cffi_d_geopm_pio_format_signal },
  { "geopm_pio_num_control_name", (void *)_cffi_f_geopm_pio_num_control_name, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 114), (void *)_cffi_d_geopm_pio_num_control_name },
  { "geopm_pio_num_signal_name", (void *)_cffi_f_geopm_pio_num_signal_name, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 114), (void *)_cffi_d_geopm_pio_num_signal_name },
  { "geopm_pio_push_control", (void *)_cffi_f_geopm_pio_push_control, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 9), (void *)_cffi_d_geopm_pio_push_control },
  { "geopm_pio_push_signal", (void *)_cffi_f_geopm_pio_push_signal, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 9), (void *)_cffi_d_geopm_pio_push_signal },
  { "geopm_pio_read_batch", (void *)_cffi_f_geopm_pio_read_batch, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 114), (void *)_cffi_d_geopm_pio_read_batch },
  { "geopm_pio_read_signal", (void *)_cffi_f_geopm_pio_read_signal, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 14), (void *)_cffi_d_geopm_pio_read_signal },
  { "geopm_pio_reset", (void *)_cffi_f_geopm_pio_reset, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 124), (void *)_cffi_d_geopm_pio_reset },
  { "geopm_pio_restore_control", (void *)_cffi_f_geopm_pio_restore_control, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 114), (void *)_cffi_d_geopm_pio_restore_control },
  { "geopm_pio_restore_control_dir", (void *)_cffi_f_geopm_pio_restore_control_dir, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 0), (void *)_cffi_d_geopm_pio_restore_control_dir },
  { "geopm_pio_sample", (void *)_cffi_f_geopm_pio_sample, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 54), (void *)_cffi_d_geopm_pio_sample },
  { "geopm_pio_save_control", (void *)_cffi_f_geopm_pio_save_control, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 114), (void *)_cffi_d_geopm_pio_save_control },
  { "geopm_pio_save_control_dir", (void *)_cffi_f_geopm_pio_save_control_dir, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 0), (void *)_cffi_d_geopm_pio_save_control_dir },
  { "geopm_pio_signal_description", (void *)_cffi_f_geopm_pio_signal_description, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 33), (void *)_cffi_d_geopm_pio_signal_description },
  { "geopm_pio_signal_domain_type", (void *)_cffi_f_geopm_pio_signal_domain_type, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 0), (void *)_cffi_d_geopm_pio_signal_domain_type },
  { "geopm_pio_signal_info", (void *)_cffi_f_geopm_pio_signal_info, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 3), (void *)_cffi_d_geopm_pio_signal_info },
  { "geopm_pio_signal_name", (void *)_cffi_f_geopm_pio_signal_name, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 83), (void *)_cffi_d_geopm_pio_signal_name },
  { "geopm_pio_start_batch_server", (void *)_cffi_f_geopm_pio_start_batch_server, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 73), (void *)_cffi_d_geopm_pio_start_batch_server },
  { "geopm_pio_stop_batch_server", (void *)_cffi_f_geopm_pio_stop_batch_server, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 51), (void *)_cffi_d_geopm_pio_stop_batch_server },
  { "geopm_pio_write_batch", (void *)_cffi_f_geopm_pio_write_batch, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 114), (void *)_cffi_d_geopm_pio_write_batch },
  { "geopm_pio_write_control", (void *)_cffi_f_geopm_pio_write_control, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 20), (void *)_cffi_d_geopm_pio_write_control },
  { "geopm_shmem_create_prof", (void *)_cffi_f_geopm_shmem_create_prof, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 38), (void *)_cffi_d_geopm_shmem_create_prof },
  { "geopm_shmem_path_prof", (void *)_cffi_f_geopm_shmem_path_prof, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 26), (void *)_cffi_d_geopm_shmem_path_prof },
  { "geopm_stats_collector_create", (void *)_cffi_f_geopm_stats_collector_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 92), (void *)_cffi_d_geopm_stats_collector_create },
  { "geopm_stats_collector_free", (void *)_cffi_f_geopm_stats_collector_free, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 97), (void *)_cffi_d_geopm_stats_collector_free },
  { "geopm_stats_collector_report", (void *)_cffi_f_geopm_stats_collector_report, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 109), (void *)_cffi_d_geopm_stats_collector_report },
  { "geopm_stats_collector_report_yaml", (void *)_cffi_f_geopm_stats_collector_report_yaml, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 104), (void *)_cffi_d_geopm_stats_collector_report_yaml },
  { "geopm_stats_collector_reset", (void *)_cffi_f_geopm_stats_collector_reset, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 97), (void *)_cffi_d_geopm_stats_collector_reset },
  { "geopm_stats_collector_update", (void *)_cffi_f_geopm_stats_collector_update, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 97), (void *)_cffi_d_geopm_stats_collector_update },
  { "geopm_stats_collector_update_count", (void *)_cffi_f_geopm_stats_collector_update_count, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 100), (void *)_cffi_d_geopm_stats_collector_update_count },
  { "geopm_topo_create_cache", (void *)_cffi_f_geopm_topo_create_cache, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 114), (void *)_cffi_d_geopm_topo_create_cache },
  { "geopm_topo_domain_idx", (void *)_cffi_f_geopm_topo_domain_idx, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 62), (void *)_cffi_d_geopm_topo_domain_idx },
  { "geopm_topo_domain_name", (void *)_cffi_f_geopm_topo_domain_name, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 83), (void *)_cffi_d_geopm_topo_domain_name },
  { "geopm_topo_domain_nested", (void *)_cffi_f_geopm_topo_domain_nested, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 66), (void *)_cffi_d_geopm_topo_domain_nested },
  { "geopm_topo_domain_type", (void *)_cffi_f_geopm_topo_domain_type, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 0), (void *)_cffi_d_geopm_topo_domain_type },
  { "geopm_topo_num_domain", (void *)_cffi_f_geopm_topo_num_domain, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 51), (void *)_cffi_d_geopm_topo_num_domain },
  { "geopm_topo_num_domain_nested", (void *)_cffi_f_geopm_topo_num_domain_nested, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 62), (void *)_cffi_d_geopm_topo_num_domain_nested },
};

static const struct _cffi_field_s _cffi_fields[] = {
  { "name", offsetof(struct geopm_metric_stats_s, name),
            sizeof(((struct geopm_metric_stats_s *)0)->name),
            _CFFI_OP(_CFFI_OP_NOOP, 127) },
  { "stats", offsetof(struct geopm_metric_stats_s, stats),
             sizeof(((struct geopm_metric_stats_s *)0)->stats),
             _CFFI_OP(_CFFI_OP_NOOP, 131) },
  { "host", offsetof(struct geopm_report_s, host),
            sizeof(((struct geopm_report_s *)0)->host),
            _CFFI_OP(_CFFI_OP_NOOP, 127) },
  { "sample_time_first", offsetof(struct geopm_report_s, sample_time_first),
                         sizeof(((struct geopm_report_s *)0)->sample_time_first),
                         _CFFI_OP(_CFFI_OP_NOOP, 127) },
  { "sample_stats", offsetof(struct geopm_report_s, sample_stats),
                    sizeof(((struct geopm_report_s *)0)->sample_stats),
                    _CFFI_OP(_CFFI_OP_NOOP, 129) },
  { "num_metric", offsetof(struct geopm_report_s, num_metric),
                  sizeof(((struct geopm_report_s *)0)->num_metric),
                  _CFFI_OP(_CFFI_OP_NOOP, 30) },
  { "metric_stats", offsetof(struct geopm_report_s, metric_stats),
                    sizeof(((struct geopm_report_s *)0)->metric_stats),
                    _CFFI_OP(_CFFI_OP_NOOP, 137) },
  { "domain_type", offsetof(struct geopm_request_s, domain_type),
                   sizeof(((struct geopm_request_s *)0)->domain_type),
                   _CFFI_OP(_CFFI_OP_NOOP, 11) },
  { "domain_idx", offsetof(struct geopm_request_s, domain_idx),
                  sizeof(((struct geopm_request_s *)0)->domain_idx),
                  _CFFI_OP(_CFFI_OP_NOOP, 11) },
  { "name", offsetof(struct geopm_request_s, name),
            sizeof(((struct geopm_request_s *)0)->name),
            _CFFI_OP(_CFFI_OP_NOOP, 127) },
};

static const struct _cffi_struct_union_s _cffi_struct_unions[] = {
  { "geopm_metric_stats_s", 138, _CFFI_F_CHECK_FIELDS,
    sizeof(struct geopm_metric_stats_s), offsetof(struct _cffi_align_struct_geopm_metric_stats_s, y), 0, 2 },
  { "geopm_report_s", 139, _CFFI_F_CHECK_FIELDS,
    sizeof(struct geopm_report_s), offsetof(struct _cffi_align_struct_geopm_report_s, y), 2, 5 },
  { "geopm_request_s", 140, _CFFI_F_CHECK_FIELDS,
    sizeof(struct geopm_request_s), offsetof(struct _cffi_align_struct_geopm_request_s, y), 7, 3 },
  { "geopm_stats_collector_s", 141, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
};

static const struct _cffi_enum_s _cffi_enums[] = {
  { "geopm_domain_e", 133, _cffi_prim_int(sizeof(enum geopm_domain_e), ((enum geopm_domain_e)-1) <= 0),
    "GEOPM_DOMAIN_INVALID,GEOPM_DOMAIN_BOARD,GEOPM_DOMAIN_PACKAGE,GEOPM_DOMAIN_CORE,GEOPM_DOMAIN_CPU,GEOPM_DOMAIN_MEMORY,GEOPM_DOMAIN_PACKAGE_INTEGRATED_MEMORY,GEOPM_DOMAIN_NIC,GEOPM_DOMAIN_PACKAGE_INTEGRATED_NIC,GEOPM_DOMAIN_GPU,GEOPM_DOMAIN_PACKAGE_INTEGRATED_GPU,GEOPM_DOMAIN_GPU_CHIP,GEOPM_NUM_DOMAIN" },
  { "geopm_error_e", 134, _cffi_prim_int(sizeof(enum geopm_error_e), ((enum geopm_error_e)-1) <= 0),
    "GEOPM_ERROR_RUNTIME,GEOPM_ERROR_LOGIC,GEOPM_ERROR_INVALID,GEOPM_ERROR_FILE_PARSE,GEOPM_ERROR_LEVEL_RANGE,GEOPM_ERROR_NOT_IMPLEMENTED,GEOPM_ERROR_PLATFORM_UNSUPPORTED,GEOPM_ERROR_MSR_OPEN,GEOPM_ERROR_MSR_READ,GEOPM_ERROR_MSR_WRITE,GEOPM_ERROR_AGENT_UNSUPPORTED,GEOPM_ERROR_AFFINITY,GEOPM_ERROR_NO_AGENT" },
  { "geopm_metric_stats_e", 135, _cffi_prim_int(sizeof(enum geopm_metric_stats_e), ((enum geopm_metric_stats_e)-1) <= 0),
    "GEOPM_METRIC_COUNT,GEOPM_METRIC_FIRST,GEOPM_METRIC_LAST,GEOPM_METRIC_MIN,GEOPM_METRIC_MAX,GEOPM_METRIC_MEAN,GEOPM_METRIC_STD,GEOPM_NUM_METRIC_STATS" },
  { "geopm_sample_stats_e", 136, _cffi_prim_int(sizeof(enum geopm_sample_stats_e), ((enum geopm_sample_stats_e)-1) <= 0),
    "GEOPM_SAMPLE_TIME_TOTAL,GEOPM_SAMPLE_COUNT,GEOPM_SAMPLE_PERIOD_MEAN,GEOPM_SAMPLE_PERIOD_STD,GEOPM_NUM_SAMPLE_STATS" },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  _cffi_fields,
  _cffi_struct_unions,
  _cffi_enums,
  NULL,  /* no typenames */
  83,  /* num_globals */
  4,  /* num_struct_unions */
  4,  /* num_enums */
  0,  /* num_typenames */
  NULL,  /* no includes */
  144,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit__libgeopmd_py_cffi(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit__libgeopmd_py_cffi(void) { return NULL; }
#  else
     init_libgeopmd_py_cffi(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit__libgeopmd_py_cffi(void)
{
  return _cffi_init("_libgeopmd_py_cffi", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
init_libgeopmd_py_cffi(void)
{
  _cffi_init("_libgeopmd_py_cffi", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
